#include <stdio.h>	// printf es hasonlok...
#include <stdlib.h>	// malloc, free
#include <string.h>	// strcmp, strcpy

// Kell egy struktura, ami a dinamikus lista egy eleme
struct elemtipus {
	// Nev: itt taroljuk az egeszet, igy helyet is kerunk neki, es
	//	nem csak egy pointer lesz a dinamikus lista egy elemeben, hogy hol van.
	//	Igy nem kell neki kulon majd helyet foglalgatni...
	char nev[40];
	int kor;	// Az eletkor sima szam, annak is itt lesz nyilvan a helye.
	struct elemtipus *kov;	// "Hol a kovetkezo elem?" Ha nincs olya, akkor NULL.
};

// Uj elem letrehozasa, kitoltese a parameterkent kapott adatokkal,
//	majd befuzese a lista _elejere_, mert az konnyebb. (Nem kell vegigmenni
//	rajta es megkeresni a veget.)
// Visszateresi ertek: a lista uj eleje
struct elemtipus *add(struct elemtipus *eleje, char *ujnev, int ujkor)
{
	// Memoriafoglalas
	struct elemtipus *ujelem = (struct elemtipus *)malloc(sizeof(struct elemtipus));
	if (!ujelem) {	// Sikerult?
		printf("Nem sikerult... :(\n");
		return eleje;	// Marad az eddigi eleje
	}
	// Ertekadasok az uj elemnek
	strcpy(ujelem->nev,ujnev);	// Masolja az "ujnev" altal mutatott
								// karaktertombot az "ujelem->nev" altal mutatott helyre.
	ujelem->kor = ujkor;
	// Dinamikus lista rendbe rakasa
	ujelem->kov = eleje;	// Befuzes
	return ujelem;		// Most mar ez a lista uj eleme
}

// Adott nevu ember torlese
// Visszateresi ertek: a lista uj eleme (hatha az elsot kellett torolni).
struct elemtipus *del(struct elemtipus *eleje, char *torlendonev)
{
	// Kell tudni, melyik elem volt az elozo, hogy ha torolni kell azt, amin
	//	allunk, akkor az elozonek a "kov"-jet at kell allitani!
	struct elemtipus *elozo = NULL;	// Mi volt az elozo?
	struct elemtipus *akt = eleje;	// Ez mutatja majd, hogy eppen hol tartunk
	while (akt) {	// Amig nem ertunk a lista vegere...
		// Az aktualis elem "nev"-je az, amit torolni akarunk?
		if (!strcmp(akt->nev,torlendonev)) {
			// Ha igen, akkor ket lehetoseg van: vagy az elso elem, vagy nem...
			if (elozo == NULL) {	// Elso elemet toroljuk
				eleje = akt->kov;	// A lista uj eleje az, ami a torlendo (elso)
									//	utan kovetkezik.
				free(akt);			// A regi eleje ("eleje" mar nem oda mutat!)
				return eleje;		// Visszaadjuk az uj elejet
			}
			else {	// Nem az elso elem
				elozo->kov = akt->kov;	// Kifuzes
				free(akt);	// Memoria felszabaditasa (szigoruan a kifuzes utan,
							//	kulonben az "akt->kov" valtozot elveszitjuk!
				return eleje;
			}
		}
		// Ha nem ez a keresett elem, akkor megyunk tovabb.
		elozo = akt;	// Az elozo az lesz, amin most allunk
		akt = akt->kov;	// Es lepunk tovabb az aktualis utani elemre...
	}
	// A lisra vegere ertunk es egyszer sem talalkoztunk a keresett nevvel.
	//	(Ha talalkoztunk volna, akkor egy "return"-nel mar veget ert volna
	//	ez a fuggveny.)
	printf("Nem talaltam meg: %s\n",torlendonev);
	return eleje;	// A lista eleje nem valtozott ebben az esetben...
}

// Irjuk ki a listaban szereplo emberek nevet es korat
// Vissza nem ad semmit, mert nem is modosit semmit, csak olvas.
void list(struct elemtipus *eleje)
{
	struct elemtipus *akt = NULL;	// Itt taroljuk, hogy eppen hol tartunk
	// For ciklus:
	// - A lista elejerol indulunk
	// - Addig megyunk, amig letezo listaelemen vagyunk
	//	 (ha leszaladtunk a vegen, megallunk)
	// - Szepen mindig lepunk a kovetkezo elemre...
	for (akt=eleje; akt!=NULL; akt=akt->kov) {
		printf("Nev: %s, kor: %d\n",akt->nev,akt->kor);	// Kiiras
	}
}

// Egesz lista minden elemenek torlese
// Visszateresi ertek: a (nem letezo) lista uj eleje: NULL
// Ez nem kell feltetlenul, de igy a foprogramban ugyanugy
// tudjuk hasznalni, mint a tobbi hasonlo fuggvenyt es nem kell
// figyelni ra, hogy utana NULL-ra allitsuk a lista elejere mutato pointert.
// (Ezzel elkerulunk egy veszelyforrast...)
struct elemtipus *clean(struct elemtipus *eleje)
{
	struct elemtipus *akt = eleje;	// Ezzel setalunk majd vegig az elemeken.
	struct elemtipus *kovetkezo = NULL;	// Mindig megjegyezzuk majd, hogy mi kovetkezik...
	while (akt) {	// Amig letezo listaelemen allunk...
		kovetkezo = akt->kov;	// Megjegyezzuk, mi jon ezutan,
		free(akt);				// mert itt meg toroltuk "akt->kov"-ot is.
		akt = kovetkezo;		// De ha tovabb akarunk lepni, akkor szuksegunk van ra,
								// ezert mentettuk el az elobb a "free" elott.
	}
	return NULL;	// A lista uj eleje... (ami mar nincs)
}

// No akkor most probaljuk ki az egesz alkotast...
int main()
{
	struct elemtipus *start = NULL;	// Valahol tarulnunk kell a lista elejet. (Ami meg nincs.)
	printf("--- Felveszek 3 elemet ---\n");	// Csak hogy a felhasznalo lassa, mi jon...
	// Felveszunk elemeket. Ha igy irjuk meg, a lista eleje mindig szepen frissul...
	start = add(start,"Elso",10);	// Ha esetleg nem sikerul a "malloc", gyakorlatilag
									// semmit nem tortenik: nem kerul be az uj elem, de
									// megyunk tovabb...
	start = add(start,"Masodik",20);
	start = add(start,"Harmadik",30);
	// Mutassuk meg a felhasznalonak, hogy mi van a listaban.
	// Mivel mindig az elejere fuzzuk az uj elemet, a sorrend forditott lesz.
	list(start);

	// Most akkor probaljuk ki a torlest...
	printf("--- Torlom 'Masodik'-at ---\n");
	start = del(start,"Masodik"); // Ha kell, "start" szepen frissul...
	list(start);	// Es lassuk az eredmenyt...

	// Kiprobaljuk, mi van, ha nem letezo embert akarunk torolni, hehehe. :)
	printf("--- Probalok 'NincsIlyen'-t torolni ---\n");
	start = del(start,"NincsIlyen");
	list(start);

	// Most ujra felveszunk egy uj elemet.
	// Ezzel kiprobaljuk, hogy sikeres es sikertelen torles utan megy-e a hozzaadas.
	printf("--- Felveszek 'Negyedik' elemet ---\n");
	start = add(start,"Negyedik",40);
	list(start);
	
	// Aztan a vegen torlunk mindent, ahogy illik.
	printf("--- Torlok mindent ---\n");
	start = clean(start);
	list(start);	// Es meg kiprobaljuk, hogy mukodik-e a listazas, ha ures a lista.
	printf("--- Kesz ---\n");
	return 0;	// Minden OK, vege a programnak...
}
